/* 
 * Copyright (C) 2006 Registro.br. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
/* $Id: CommonData.H 563 2006-03-10 17:23:49Z cesar $ */
/** @file CommonData.H
 *  @brief EPP CommonData Class
 */

#ifndef __COMMON_DATA_H__
#define __COMMON_DATA_H__

#include <string>
#include <vector>
#include <set>

#include "libepp_nicbr.H"

using std::string;
using std::vector;
using std::set;

LIBEPP_NICBR_NS_BEGIN

/// Action Types
enum ActionType {
  UNSET_ACTION = -1,
  LOGIN = 0,
  LOGOUT,
  HELLO,
  POLL,
  CONTACT_CHECK = 10,
  CONTACT_CREATE,
  CONTACT_INFO,
  CONTACT_TRANSFER,
  CONTACT_UPDATE,
  DOMAIN_CHECK = 20,
  DOMAIN_CREATE,
  DOMAIN_INFO,
  DOMAIN_RENEW,
  DOMAIN_UPDATE,
  BR_DOMAIN_CHECK = 30,
  BR_DOMAIN_INFO,
  BR_DOMAIN_CREATE,
  BR_DOMAIN_UPDATE,
  BR_ORG_CHECK = 40,
  BR_ORG_INFO,
  BR_ORG_CREATE,
  BR_ORG_UPDATE,
  DOMAIN_PANDATA = 50,
  CONTACT_PANDATA,
  BR_ORG_PANDATA = 60
};

/// AuthInfo Class
class AuthInfo {
public:
  AuthInfo& operator=(const AuthInfo &authInfo) {
    _roid = authInfo._roid;
    _pw = authInfo._pw;
    return *this;
  }

  /// Default constructor
  AuthInfo()
  {
    this->reset();
  }

  /// Sets repository object ID
  /**
     @param roid   repository object id
   */
  void set_roid(string roid) { _roid = roid; }

  /// Returns repository object ID
  /**
     @return repository object ID
   */
  string get_roid() { return _roid; }

  /// Sets password
  /**
     @param pw   password
   */
  void set_pw(string pw) { _pw = pw; }

  /// Returns password
  /**
     @return password
   */
  string get_pw() { return _pw; }

  /// Reset all object attributes
  void reset()
  {
    _roid = "";
    _pw = "";
  }

protected:
  /// repository object ID
  string _roid;

  /// repository object ID
  string _pw;
};

struct NSIPAddr {
  string version;
  string addr;
  bool operator<(const NSIPAddr &ip) const {
    return addr < ip.addr;
  }
};

struct NameServer {
  string name;
  set<NSIPAddr> ips;
  bool operator<(const NameServer &n) const {
    return name < n.name;
  }
};

/// EPP CommonData Class
class CommonData
{
public:
  
  struct PostalInfo {
    string type;
    string name;
    string org;
    string str1;
    string str2;
    string str3;
    string city;
    string sp;
    string pc;
    string cc;
  };

  struct Disclose {
    bool flag;
    bool name_int;
    bool name_loc;
    bool org_int;
    bool org_loc;
    bool addr_int;
    bool addr_loc;
    bool voice;
    bool fax;
    bool email;
    Disclose() {
      flag = false;
      name_int = false;
      name_loc = false;
      org_int = false;
      org_loc = false;
      addr_int = false;
      addr_loc = false;
      voice = false;
      fax = false;
      email = false;
    }

    bool is_set() {      
      return (name_int || name_loc || org_int || org_loc || addr_int || 
              addr_loc || voice || fax || email);
    }
  };

  struct Phone {
    string ext;
    string number;
  };


  /// Null Constructor
  CommonData() {}  

  /// Sets the object's id
  /**
     @param id contact id
  */
  void set_id(const string &id);
  
  /// Inserts postal information into the object
  /**
    @param postal_info postal information
  */
  void insert_postal_info(const PostalInfo& postal_info);

  /// Sets the object's voice telephone number
  /**
     @param voice contact voice telephone
  */
  void set_voice(const Phone& voice);
  
  /// Sets the object's fax number
  /**
     @param  fax contact fax 
  */
  void set_fax(const Phone& fax);
  
  /// Sets the object's email
  /**
     @param email contact email
  */
  void set_email(const string &email);
  
  /// Sets the object's disclosure policy
  /**
     @param disclose disclose information
  */
  void set_disclose(const Disclose &disclose);
  
  /// Returns the object's id
  /**
     @return Contact id
  */
  string get_id();
  
  /// Returns the object's postal information
  /**
     @return Postal info list
  */
  vector<PostalInfo> get_postal_info();

  /// Returns the object's voice telephone number
  /**
     @return voice contact voice telephone number
  */
  Phone get_voice();
  
  /// Returns the object's fax number
  /**
     @return fax contact fax number
  */
  Phone get_fax();
  
  /// Returns the object's email
  /**
     @return email contact email
  */
  string get_email();
  
  /// Returns the object's disclosure policy
  /**
     @return disclose disclose information
  */
  Disclose get_disclose();

  /// reset attributes
  void reset();
  
protected:
  
  /// Contact id 
  string _id;

  /// Postal Info list (just 1 or 2 elements)
  vector<PostalInfo> _postal_info;

  /// Contact's voice telephone number (optional)
  Phone _voice;

  /// Contact's fax (optional)
  Phone _fax;

  /// Contact's email
  string _email;

  /// Disclose information (optional)
  Disclose _disclose;
};

LIBEPP_NICBR_NS_END
#endif //__COMMON_DATA_H__
